import { getDefaultThrottle, debug, parseAsString, parseAsInteger, parseAsFloat, parseAsBoolean, parseAsTimestamp, parseAsIsoDateTime, parseAsStringEnum, parseAsJson, parseAsArrayOf, safeParse } from './chunk-RFOPYEUW.js';
export { createParser, parseAsArrayOf, parseAsBoolean, parseAsFloat, parseAsHex, parseAsInteger, parseAsIsoDateTime, parseAsJson, parseAsString, parseAsStringEnum, parseAsTimestamp } from './chunk-RFOPYEUW.js';
import Mitt from 'mitt';
import { useRouter, useSearchParams } from 'next/navigation.js';
import React from 'react';

// src/deprecated.ts
var queryTypes = {
  /**
   * @deprecated use `parseAsString` instead.
   */
  string: parseAsString,
  /**
   * @deprecated use `parseAsInteger` instead.
   */
  integer: parseAsInteger,
  /**
   * @deprecated use `parseAsFloat` instead.
   */
  float: parseAsFloat,
  /**
   * @deprecated use `parseAsBoolean` instead.
   */
  boolean: parseAsBoolean,
  /**
   * @deprecated use `parseAsTimestamp` instead.
   */
  timestamp: parseAsTimestamp,
  /**
   * @deprecated use `parseAsIsoDateTime` instead.
   */
  isoDateTime: parseAsIsoDateTime,
  /**
   * @deprecated use `parseAsStringEnum` instead.
   */
  stringEnum: parseAsStringEnum,
  /**
   * @deprecated use `parseAsJson` instead.
   */
  json: parseAsJson,
  /**
   * @deprecated use `parseAsArrayOf` instead.
   */
  array: parseAsArrayOf
};

// src/errors.ts
var errors = {
  409: "Multiple versions of the library are loaded. This may lead to unexpected behavior. Currently using %s, but %s was about to load on top.",
  429: "URL update rate-limited by the browser. Consider increasing `throttleMs` for keys %s. %O"
};
function error(code, ...args) {
  const message = `[next-usequerystate] ${errors[code]}
  See https://err.47ng.com/NUQS-${code}`;
  console.error(message, ...args);
}

// src/url-encoding.ts
function renderQueryString(search) {
  const query = [];
  for (const [key, value] of search.entries()) {
    query.push(`${key}=${encodeQueryValue(value)}`);
  }
  return query.join("&");
}
function encodeQueryValue(input) {
  return input.replace(/%/g, "%25").replace(/\+/g, "%2B").replace(/ /g, "+").replace(/#/g, "%23").replace(/&/g, "%26").replace(/"/g, "%22").replace(/'/g, "%27").replace(/`/g, "%60").replace(/</g, "%3C").replace(/>/g, "%3E");
}

// src/update-queue.ts
var FLUSH_RATE_LIMIT_MS = getDefaultThrottle();
var updateQueue = /* @__PURE__ */ new Map();
var queueOptions = {
  history: "replace",
  scroll: false,
  shallow: true,
  throttleMs: FLUSH_RATE_LIMIT_MS
};
var lastFlushTimestamp = 0;
var flushPromiseCache = null;
function enqueueQueryStringUpdate(key, value, serialize, options) {
  const serializedOrNull = value === null ? null : serialize(value);
  debug("[nuqs queue] Enqueueing %s=%s %O", key, serializedOrNull, options);
  updateQueue.set(key, serializedOrNull);
  if (options.history === "push") {
    queueOptions.history = "push";
  }
  if (options.scroll) {
    queueOptions.scroll = true;
  }
  if (options.shallow === false) {
    queueOptions.shallow = false;
  }
  queueOptions.throttleMs = Math.max(
    options.throttleMs ?? FLUSH_RATE_LIMIT_MS,
    Number.isFinite(queueOptions.throttleMs) ? queueOptions.throttleMs : 0
  );
}
function getQueuedValue(key) {
  return updateQueue.get(key) ?? null;
}
function scheduleFlushToURL(router) {
  if (flushPromiseCache === null) {
    flushPromiseCache = new Promise((resolve, reject) => {
      if (!Number.isFinite(queueOptions.throttleMs)) {
        debug("[nuqs queue] Skipping flush due to throttleMs=Infinity");
        resolve(new URLSearchParams(location.search));
        setTimeout(() => {
          flushPromiseCache = null;
        }, 0);
        return;
      }
      function flushNow() {
        lastFlushTimestamp = performance.now();
        const [search, error2] = flushUpdateQueue(router);
        if (error2 === null) {
          resolve(search);
        } else {
          reject(search);
        }
        flushPromiseCache = null;
      }
      function runOnNextTick() {
        const now = performance.now();
        const timeSinceLastFlush = now - lastFlushTimestamp;
        const throttleMs = queueOptions.throttleMs;
        const flushInMs = Math.max(
          0,
          Math.min(throttleMs, throttleMs - timeSinceLastFlush)
        );
        debug(
          "[nuqs queue] Scheduling flush in %f ms. Throttled at %f ms",
          flushInMs,
          throttleMs
        );
        if (flushInMs === 0) {
          flushNow();
        } else {
          setTimeout(flushNow, flushInMs);
        }
      }
      setTimeout(runOnNextTick, 0);
    });
  }
  return flushPromiseCache;
}
function flushUpdateQueue(router) {
  const search = new URLSearchParams(location.search);
  if (updateQueue.size === 0) {
    return [search, null];
  }
  const items = Array.from(updateQueue.entries());
  const options = { ...queueOptions };
  updateQueue.clear();
  queueOptions.history = "replace";
  queueOptions.scroll = false;
  queueOptions.shallow = true;
  queueOptions.throttleMs = FLUSH_RATE_LIMIT_MS;
  debug("[nuqs queue] Flushing queue %O with options %O", items, options);
  for (const [key, value] of items) {
    if (value === null) {
      search.delete(key);
    } else {
      search.set(key, value);
    }
  }
  const url = renderURL(search);
  debug("[nuqs queue] Updating url: %s", url);
  try {
    const updateMethod = options.history === "push" ? history.pushState : history.replaceState;
    updateMethod.call(
      history,
      history.state,
      // Our own updates have a marker to prevent syncing
      // when the URL changes (we've already sync'd them up
      // via `emitter.emit(key, newValue)` above, without
      // going through the parsers).
      NOSYNC_MARKER,
      url
    );
    if (options.scroll) {
      window.scrollTo(0, 0);
    }
    if (!options.shallow) {
      router.replace(url, {
        scroll: false,
        // @ts-expect-error - pages router fix, but not exposed in navigation types
        shallow: false
      });
    }
    return [search, null];
  } catch (err) {
    error(429, items.map(([key]) => key).join(), err);
    return [search, error];
  }
}
function renderURL(search) {
  const query = renderQueryString(search);
  const href = location.href.split("?")[0];
  const hash = location.hash;
  if (history.state.__N === true) {
    const basePath = window?.next?.router?.basePath ?? "";
    const path = location.pathname.slice(basePath.length);
    const base = process.env.__NEXT_WINDOW_HISTORY_SUPPORT ? href : path;
    return query ? `${base}?${query}${hash}` : `${base}${hash}`;
  } else {
    return query ? `${href}?${query}${hash}` : `${href}${hash}`;
  }
}

// src/sync.ts
var SYNC_EVENT_KEY = Symbol("__nextUseQueryState__SYNC__");
var NOSYNC_MARKER = "__nextUseQueryState__NO_SYNC__";
var NOTIFY_EVENT_KEY = Symbol("__nextUseQueryState__NOTIFY__");
var emitter = Mitt();
function subscribeToQueryUpdates(callback) {
  emitter.on(NOTIFY_EVENT_KEY, callback);
  return () => emitter.off(NOTIFY_EVENT_KEY, callback);
}
if (typeof history === "object") {
  patchHistory();
}
function patchHistory() {
  const version = "1.12.0";
  const patched = history.__nextUseQueryState_patched;
  if (patched) {
    if (patched !== version) {
      error(409, patched, version);
    }
    return;
  }
  debug("[nuqs] Patching history with %s", version);
  for (const method of ["pushState", "replaceState"]) {
    const original = history[method].bind(history);
    history[method] = function nextUseQueryState_patchedHistory(state, title, url) {
      if (!url) {
        debug("[nuqs] history.%s(null) (%s) %O", method, title, state);
        return original(state, title, url);
      }
      const source = title === NOSYNC_MARKER ? "internal" : "external";
      const search = new URL(url, location.origin).searchParams;
      debug(`[nuqs] history.%s(%s) (%s) %O`, method, url, source, state);
      if (source === "external") {
        for (const [key, value] of search.entries()) {
          const queueValue = getQueuedValue(key);
          if (queueValue !== null && queueValue !== value) {
            debug(
              "[nuqs] Overwrite detected for key: %s, Server: %s, queue: %s",
              key,
              value,
              queueValue
            );
            search.set(key, queueValue);
          }
        }
        setTimeout(() => {
          debug(
            "[nuqs] External history.%s call: triggering sync with %s",
            method,
            search
          );
          emitter.emit(SYNC_EVENT_KEY, search);
          emitter.emit(NOTIFY_EVENT_KEY, { search, source });
        }, 0);
      } else {
        setTimeout(() => {
          emitter.emit(NOTIFY_EVENT_KEY, { search, source });
        }, 0);
      }
      return original(state, title === NOSYNC_MARKER ? "" : title, url);
    };
  }
  Object.defineProperty(history, "__nextUseQueryState_patched", {
    value: version,
    writable: false,
    enumerable: false,
    configurable: false
  });
}
function useQueryState(key, {
  history: history2 = "replace",
  shallow = true,
  scroll = false,
  throttleMs = FLUSH_RATE_LIMIT_MS,
  parse = (x) => x,
  serialize = String,
  defaultValue = void 0
} = {
  history: "replace",
  scroll: false,
  shallow: true,
  throttleMs: FLUSH_RATE_LIMIT_MS,
  parse: (x) => x,
  serialize: String,
  defaultValue: void 0
}) {
  const router = useRouter();
  const initialSearchParams = useSearchParams();
  const [internalState, setInternalState] = React.useState(() => {
    const queueValue = getQueuedValue(key);
    const urlValue = typeof location !== "object" ? (
      // SSR
      initialSearchParams?.get(key) ?? null
    ) : (
      // Components mounted after page load must use the current URL value
      new URLSearchParams(location.search).get(key) ?? null
    );
    const value = queueValue ?? urlValue;
    return value === null ? null : safeParse(parse, value);
  });
  const stateRef = React.useRef(internalState);
  debug(
    "[nuqs `%s`] render - state: %O, iSP: %s",
    key,
    internalState,
    initialSearchParams?.get(key) ?? null
  );
  if (process.env.__NEXT_WINDOW_HISTORY_SUPPORT) {
    React.useEffect(() => {
      const value = initialSearchParams.get(key) ?? null;
      const state = value === null ? null : parse(value);
      debug(`[nuqs \`%s\`] syncFromUseSearchParams %O`, key, state);
      stateRef.current = state;
      setInternalState(state);
    }, [initialSearchParams?.get(key), key]);
  }
  React.useInsertionEffect(() => {
    function updateInternalState(state) {
      debug(`[nuqs \`%s\`] updateInternalState %O`, key, state);
      stateRef.current = state;
      setInternalState(state);
    }
    function syncFromURL(search) {
      const value = search.get(key) ?? null;
      const state = value === null ? null : parse(value);
      debug(`[nuqs \`%s\`] syncFromURL %O`, key, state);
      updateInternalState(state);
    }
    debug(`[nuqs \`%s\`] subscribing to sync`, key);
    emitter.on(SYNC_EVENT_KEY, syncFromURL);
    emitter.on(key, updateInternalState);
    return () => {
      debug(`[nuqs \`%s\`] unsubscribing from sync`, key);
      emitter.off(SYNC_EVENT_KEY, syncFromURL);
      emitter.off(key, updateInternalState);
    };
  }, [key]);
  const update = React.useCallback(
    (stateUpdater, options = {}) => {
      const newValue = isUpdaterFunction(stateUpdater) ? stateUpdater(stateRef.current ?? defaultValue ?? null) : stateUpdater;
      emitter.emit(key, newValue);
      enqueueQueryStringUpdate(key, newValue, serialize, {
        // Call-level options take precedence over hook declaration options.
        history: options.history ?? history2,
        shallow: options.shallow ?? shallow,
        scroll: options.scroll ?? scroll,
        throttleMs: options.throttleMs ?? throttleMs
      });
      return scheduleFlushToURL(router);
    },
    [key, history2, shallow, scroll, throttleMs]
  );
  return [internalState ?? defaultValue ?? null, update];
}
function isUpdaterFunction(stateUpdater) {
  return typeof stateUpdater === "function";
}
function useQueryStates(keyMap, {
  history: history2 = "replace",
  scroll = false,
  shallow = true,
  throttleMs = FLUSH_RATE_LIMIT_MS
} = {}) {
  const keys = Object.keys(keyMap).join(",");
  const router = useRouter();
  const initialSearchParams = useSearchParams();
  const [internalState, setInternalState] = React.useState(() => {
    if (typeof location !== "object") {
      return parseMap(keyMap, initialSearchParams ?? new URLSearchParams());
    }
    return parseMap(keyMap, new URLSearchParams(location.search));
  });
  const stateRef = React.useRef(internalState);
  debug(
    "[nuq+ `%s`] render - state: %O, iSP: %s",
    keys,
    internalState,
    initialSearchParams
  );
  React.useInsertionEffect(() => {
    function updateInternalState(state) {
      debug("[nuq+ `%s`] updateInternalState %O", keys, state);
      stateRef.current = state;
      setInternalState(state);
    }
    function syncFromURL(search) {
      const state = parseMap(keyMap, search);
      debug("[nuq+ `%s`] syncFromURL %O", keys, state);
      updateInternalState(state);
    }
    const handlers = Object.keys(keyMap).reduce((handlers2, key) => {
      handlers2[key] = (value) => {
        const { defaultValue } = keyMap[key];
        stateRef.current = {
          ...stateRef.current,
          [key]: value ?? defaultValue ?? null
        };
        debug(
          "[nuq+ `%s`] Cross-hook key sync %s: %O (default: %O). Resolved: %O",
          keys,
          key,
          value,
          defaultValue,
          stateRef.current
        );
        updateInternalState(stateRef.current);
      };
      return handlers2;
    }, {});
    emitter.on(SYNC_EVENT_KEY, syncFromURL);
    for (const key of Object.keys(keyMap)) {
      debug("[nuq+ `%s`] Subscribing to sync for `%s`", keys, key);
      emitter.on(key, handlers[key]);
    }
    return () => {
      emitter.off(SYNC_EVENT_KEY, syncFromURL);
      for (const key of Object.keys(keyMap)) {
        debug("[nuq+ `%s`] Unsubscribing to sync for `%s`", keys, key);
        emitter.off(key, handlers[key]);
      }
    };
  }, [keyMap]);
  const update = React.useCallback(
    (stateUpdater, options = {}) => {
      const newState = typeof stateUpdater === "function" ? stateUpdater(stateRef.current) : stateUpdater;
      debug("[nuq+ `%s`] setState: %O", keys, newState);
      for (const [key, value] of Object.entries(newState)) {
        const config = keyMap[key];
        if (!config) {
          continue;
        }
        emitter.emit(key, value);
        enqueueQueryStringUpdate(key, value, config.serialize ?? String, {
          // Call-level options take precedence over hook declaration options.
          history: options.history ?? history2,
          shallow: options.shallow ?? shallow,
          scroll: options.scroll ?? scroll,
          throttleMs: options.throttleMs ?? throttleMs
        });
      }
      return scheduleFlushToURL(router);
    },
    [keyMap, history2, shallow, scroll]
  );
  return [internalState, update];
}
function parseMap(keyMap, searchParams) {
  return Object.keys(keyMap).reduce((obj, key) => {
    const { defaultValue, parse } = keyMap[key];
    const urlQuery = searchParams?.get(key) ?? null;
    const queueQuery = getQueuedValue(key);
    const query = queueQuery ?? urlQuery;
    const value = query === null ? null : safeParse(parse, query);
    obj[key] = value ?? defaultValue ?? null;
    return obj;
  }, {});
}

export { queryTypes, subscribeToQueryUpdates, useQueryState, useQueryStates };
